@@ -1,7 +1,7 @@
 ###       Functions that are used in several parts of the nlme library
 ###                 but do not belong to any specific part
 ###
-### Copyright 2006-2016  The R Core team
+### Copyright 2006-2022  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
 ###
@@ -18,6 +18,8 @@
 ###  A copy of the GNU General Public License is available at
 ###  http://www.r-project.org/Licenses/
 
+svd.d <- function(x) La.svd(x, nu=0L, nv=0L)$d
+
 allCoef <-
   ## Combines different coefficient vectors into one vector, keeping track
   ## of which coefficients came from which object
@@ -55,12 +57,11 @@ asOneFormula <-
   ## list of formulas, except for the names in omit
   function(..., omit = c(".", "pi"))
 {
-  dots <- list(...)
-  names <- unique(allVarsRec((dots)))
+  names <- unique(allVarsRec(list(...)))
   names <- names[is.na(match(names, omit))]
   if (length(names))
-    eval(parse(text = paste("~", paste(names, collapse = "+")))[[1]])
-  else ~1
+    eval(parse(text = paste("~", paste(names, collapse = "+")))[[1]], .GlobalEnv)
+  else evalq(~1, .GlobalEnv)
 }
 
 compareFits <-
@@ -96,7 +97,7 @@ compareFits <-
   out
 }
 
-fdHess <- function(pars, fun, ..., .relStep = (.Machine$double.eps)^(1/3),
+fdHess <- function(pars, fun, ..., .relStep = .Machine$double.eps^(1/3),
                    minAbsPar = 0)
   ## Use a Koschal design to establish a second order model for the response
 {
@@ -191,7 +192,7 @@ getCovariateFormula <-
   if (length(form) == 3 && form[[1]] == as.name("|")){ # conditional expression
     form <- form[[2]]
   }
-  eval(substitute(~form))
+  eval(call("~", form), .GlobalEnv)
 }
 
 getResponseFormula <-
@@ -202,7 +203,7 @@ getResponseFormula <-
   if (!(inherits(form, "formula") && (length(form) == 3))) {
     stop("'form' must be a two-sided formula")
   }
-  eval(parse(text = paste("~", deparse(form[[2]]))))
+  eval(call("~", form[[2]]), .GlobalEnv)
 }
 
 gsummary <-
@@ -247,13 +248,11 @@ gsummary <-
       aux <- table(x)
       names(aux)[match(max(aux), aux)]
     }
-    if (data.class(FUN) == "function") {	# single function given
+    if (is.function(FUN)) {	# single function given
       FUN <- list(numeric = FUN, ordered = Mode, factor = Mode)
     } else {
-      if (!(is.list(FUN) &&
-	   all(sapply(FUN, data.class) == "function"))) {
+      if (!(is.list(FUN) && all(sapply(FUN, is.function))))
 	stop("'FUN' can only be a function or a list of functions")
-      }
       auxFUN <- list(numeric = mean, ordered = Mode, factor = Mode)
       aux <- names(auxFUN)[is.na(match(names(auxFUN), names(FUN)))]
       if (length(aux) > 0) FUN[aux] <- auxFUN[aux]
